"""Working with a solution directory"""

from PyFoam.Basics.Utilities import Utilities
from PyFoam.Basics.BasicFile import BasicFile

from os import listdir,path,mkdir
import os

class SolutionDirectory(Utilities):
    """Represents a solution directory

    In the solution directory subdirectories whose names are numbers
    are assumed to be solutions for a specific time-step

    A sub-directory (called the Archive) is created to which solution
    data is copied"""
    
    def __init__(self,name,archive="ArchiveDir"):
        """name - Name of the solution directory
        archive - name of the directory where the lastToArchive-method
        should copy files"""
        self.name=name
        self.archive=path.join(name,archive)
        if not path.exists(self.archive):
            mkdir(self.archive)

        self.backups=[]
        
        self.reread()
        
    def reread(self):
        """Rescan the directory for the time directories"""
        self.times=[]
        self.first=None
        self.last=None
        
        for f in listdir(self.name):
            try:
                val=float(f)
                self.times.append(f)
                if self.first==None:
                    self.first=f
                else:
                    if float(f)<float(self.first):
                        self.first=f
                if self.last==None:
                    self.last=f
                else:
                    if float(f)>float(self.last):
                        self.last=f
                        
            except ValueError:
                pass

    def addBackup(self,pth):
        """add file to list of files that are to be copied to the
        archive"""
        self.backups.append(path.join(self.name,pth))

    def getLast(self):
        """return the last time for which a solution exists"""
        self.reread()
        return self.last
    
    def lastToArchive(self,name):
        """copy the last solution (plus the backup-files to the
        archive)

        name - name of the sub-directory in the archive"""
        self.reread()
        fname=path.join(self.archive,name)
        if path.exists(fname):
            self.execute("rm -r "+fname)
        mkdir(fname)
        self.execute("cp -r "+path.join(self.name,self.last)+" "+fname)
        for f in self.backups:
            self.execute("cp -r "+f+" "+fname)
            
    def clearResults(self):
        """remove all time-directories except for the first (initial
        condition)"""
        for f in self.times:
            if f!=self.first:
                self.execute("rm -r "+path.join(self.name,f))
                
    def initialDir(self):
        """return the name of the first time-directory (==initial
        conditions"""
        return path.join(self.name,self.first)

    def makeFile(self,name):
        """create a file in the solution directory and return a
        corresponding BasicFile-object

        name - Name of the file"""
        return BasicFile(path.join(self.name,name))
